%if 0%{?fedora}
%global with_devel 0
%global with_bundled 1
%global with_debug 1
%else
%global with_devel 0
%global with_bundled 1
%global with_debug 0
%endif

%if 0%{?with_debug}
# https://bugzilla.redhat.com/show_bug.cgi?id=995136#c12
%global _dwz_low_mem_die_limit 0
%else
%global debug_package   %{nil}
%endif

%global k8s_repo            kubernetes

#I really need this, otherwise "version_ldflags=$(kube::version_ldflags)"
# does not work
%global _buildshell	/bin/bash
%global _checkshell	/bin/bash

# k8s release version major.minor
%global k8s_major_version	1
%global k8s_minor_version       10
%global k8s_minor_minor_version 5

# GOLANG version
%global golang_version		1.10.8
%global golang_version_hash	d8626fb6f9a3ab397d88c483b576be41fa81eefcec2fd18562c87626dbb3c39e

Name:		kubernetes
Version:	%{k8s_major_version}.%{k8s_minor_version}.%{k8s_minor_minor_version}
Release:	2.2.4%{?dist}
Summary:        Container cluster management
License:        ASL 2.0
Group:          System/Management
URL:           	https://kubernetes.io
Vendor:		Oracle America
ExclusiveArch:  x86_64 ppc64le %{arm} aarch64
Source: 	kubernetes-%{version}.tar.bz2
Source31:       genmanpages.sh

Source40:	10-kubeadm.conf
Source41: 	kubelet.service
Source42:	kubeadm-setup.sh
Source43:	k8s.conf
Source44:	kubernetes-dashboard-ol.yaml
Source45:	flannel-ol.yaml
Source46:	kubectl-proxy.service
Source47:	10-kubectl-proxy.conf
Source50:	kubeadm-registry.sh

%description
%{summary}

BuildRequires: golang >= 1.9
BuildRequires: systemd
BuildRequires: rsync
BuildRequires: go-md2man
BuildRequires: git

%package -n kubelet
Summary: Container cluster management
BuildRequires: golang >= 1.9
BuildRequires: systemd
BuildRequires: curl
Requires: iptables >= 1.4.21
Requires: socat
Requires: util-linux
Requires: ethtool
Requires: iproute
Requires: ebtables

%description -n kubelet
The node agent of Kubernetes, the container cluster manager.

%package -n kubectl
Summary: Command-line utility for interacting with a Kubernetes cluster.

%description -n kubectl
Command-line utility for interacting with a Kubernetes cluster.

%package -n kubeadm
Summary: Command-line utility for administering a Kubernetes cluster.
Requires: kubelet >= %{version}-%{release}
Requires: kubectl >= %{version}-%{release}
Requires: kubernetes-cni >= 0.6.0
Requires: docker-engine >= 1.12.6
Requires: openssl

%description -n kubeadm
Command-line utility for administering a Kubernetes cluster.

%package -n kubeadm-docker-images
Summary: Contains Oracle built k8s docker images

%description -n kubeadm-docker-images
Contains Oracle built k8s docker images

%prep
%setup -q -n %{k8s_repo}-%{version}

dirs=$(ls | grep -v "^Godeps")

# Move all the code under src/k8s.io/kubernetes directory
mkdir -p src/k8s.io/kubernetes
mv $(ls | grep -v "^src$") src/k8s.io/kubernetes/.

%build
pushd src/k8s.io/kubernetes/
export KUBE_EXTRA_GOPATH=$(pwd)/Godeps/_workspace

export GOPATH=$(pwd)/Godeps/_workspace
#export GOBIN=$GOPATH/bin
#export PATH=$GOBIN:$PATH
export KUBE_GIT_TREE_STATE=clean
export KUBE_GIT_VERSION=v%{version}+%{release}
export KUBE_GIT_MAJOR=%{k8s_major_version}
export KUBE_GIT_MINOR=%{k8s_minor_version}
# GOLANG use upstream
mkdir -p $(pwd)/.gobin
#curl -fSL https://golang.org/dl/go%{golang_version}.linux-amd64.tar.gz | tar xzC $(pwd)/.gobin
GO_TGT=go%{golang_version}.linux-amd64.tar.gz
GO_SUM=%{golang_version_hash}
curl -fSL https://golang.org/dl/${GO_TGT} > ${GO_TGT}
CHKSUM=$(sha256sum ${GO_TGT} | cut -d' ' -f1)
if [[ "${CHKSUM}" != "${GO_SUM}" ]]; then
	echo "golang upstream binary checksum doesn't match"
	exit 1
fi
tar xzC $(pwd)/.gobin < ${GO_TGT}
export GOROOT=$(pwd)/.gobin/go
export GOBIN=$(pwd)/.gobin/go/bin
export PATH=$GOBIN:$PATH

go version
go get -u github.com/jteeuwen/go-bindata/go-bindata

make WHAT='--use_go_build cmd/kube-proxy cmd/kube-apiserver cmd/kube-controller-manager cmd/kubelet cmd/hyperkube cmd/kube-scheduler cmd/kubectl cmd/kubeadm'

./build-k8s-docker.sh %{version} _output/bin/ %{golang_version} %{golang_version_hash}
# convert md to man
pushd docs
pushd admin
cp kube-apiserver.md kube-controller-manager.md kube-proxy.md kube-scheduler.md kubelet.md ..
popd
cp %{SOURCE31} genmanpages.sh
bash genmanpages.sh
popd
popd

%install
pushd src/k8s.io/kubernetes/
. hack/lib/init.sh
kube::golang::setup_env

%ifarch ppc64le
output_path="${KUBE_OUTPUT_BINPATH}"
%else
output_path="${KUBE_OUTPUT_BINPATH}/linux/amd64"
%endif

#binaries=(kube-apiserver kube-controller-manager kube-scheduler kube-proxy kubelet kubectl hyperkube kubeadm)
binaries=(kubelet kubectl kubeadm)
install -m 755 -d %{buildroot}%{_bindir}
for bin in "${binaries[@]}"; do
  echo "+++ INSTALLING ${bin}"
  install -p -m 755 -t %{buildroot}%{_bindir} ${output_path}/${bin}
done

# install the bash completion
install -d -m 0755 %{buildroot}%{_datadir}/bash-completion/completions/
%{buildroot}%{_bindir}/kubectl completion bash > %{buildroot}%{_datadir}/bash-completion/completions/kubectl

# install manpages
install -d %{buildroot}%{_mandir}/man1
install -p -m 644 docs/man/man1/kubectl* %{buildroot}%{_mandir}/man1
install -p -m 644 docs/man/man1/kubelet* %{buildroot}%{_mandir}/man1
install -p -m 644 docs/man/man1/kubeadm* %{buildroot}%{_mandir}/man1
# from k8s tarball copied docs/man/man1/*.1

# EXTRA FOR KUBEADM/KUBELET
install -m 755 -d %{buildroot}%{_sysconfdir}/systemd/system/kubelet.service.d/
install -p -m 644 -t %{buildroot}%{_sysconfdir}/systemd/system/kubelet.service.d %{SOURCE40}
install -m 755 -d %{buildroot}%{_sysconfdir}/systemd/system/kubectl-proxy.service.d/
install -p -m 644 -t %{buildroot}%{_sysconfdir}/systemd/system/kubectl-proxy.service.d %{SOURCE47}

install -m 755 -d %{buildroot}%{_sysconfdir}/kubernetes/manifests/
install -m 755 -d %{buildroot}%{_sysconfdir}/systemd/system/
install -m 755 -d %{buildroot}/usr/libexec/kubernetes/kubelet-plugins/volume/exec/

install -p -m 644 -t %{buildroot}%{_sysconfdir}/systemd/system/ %{SOURCE41}
install -p -m 644 -t %{buildroot}%{_sysconfdir}/systemd/system/ %{SOURCE46}
install -p -m 755 -t %{buildroot}%{_bindir} %{SOURCE42}
install -p -m 755 -t %{buildroot}%{_bindir} %{SOURCE50}
install -d -m 755 %{buildroot}/etc/sysctl.d
install -p -m 644 -t %{buildroot}/etc/sysctl.d %{SOURCE43}

# EXTRA FOR DOCKER IMAGES
mkdir -p %{buildroot}/usr/local/share/kubeadm
install -m 755 -d %{buildroot}/usr/local/share/kubeadm
images=(kube-apiserver.tar kube-controller-manager.tar kube-scheduler.tar kube-proxy.tar pause-amd64.tar)
for bin in "${images[@]}"; do
  echo "+++ INSTALLING DOCKER IMAGES ${bin}"
  install -p -m 755 -t %{buildroot}/usr/local/share/kubeadm ${output_path}/oracle_docker/${bin}
done

# INSTALL K8S RELATED YAML
install -p -m 644 -t %{buildroot}/usr/local/share/kubeadm %{SOURCE44}
install -p -m 644 -t %{buildroot}/usr/local/share/kubeadm %{SOURCE45}

popd
mv src/k8s.io/kubernetes/*.md .
mv src/k8s.io/kubernetes/LICENSE .

%check
#define license tag if not already defined
%{!?_licensedir:%global license %doc}

%files -n kubelet
%license LICENSE
%{_bindir}/kubelet
%{_sysconfdir}/systemd/system/kubelet.service
%{_sysconfdir}/systemd/system/kubelet.service.d/10-kubeadm.conf
%{_sysconfdir}/kubernetes/manifests/
/usr/libexec/kubernetes/kubelet-plugins/volume/exec
%doc *.md
%{_mandir}/man1/kubelet*

%files -n kubeadm
%license LICENSE
%doc *.md
%{_mandir}/man1/kubeadm*
%{_bindir}/kubeadm
%{_bindir}/kubeadm-setup.sh
%{_bindir}/kubeadm-registry.sh
/etc/sysctl.d/k8s.conf
/usr/local/share/kubeadm/kubernetes-dashboard-ol.yaml
/usr/local/share/kubeadm/flannel-ol.yaml

%files -n kubectl
%license LICENSE
%{_bindir}/kubectl
%{_sysconfdir}/systemd/system/kubectl-proxy.service.d/10-kubectl-proxy.conf
%{_sysconfdir}/systemd/system/kubectl-proxy.service
%doc *.md
%{_mandir}/man1/kubectl*
%{_datadir}/bash-completion/completions/kubectl

%files -n kubeadm-docker-images
%license LICENSE
/usr/local/share/kubeadm/kube-apiserver.tar
/usr/local/share/kubeadm/kube-controller-manager.tar
/usr/local/share/kubeadm/kube-scheduler.tar
/usr/local/share/kubeadm/kube-proxy.tar
/usr/local/share/kubeadm/pause-amd64.tar

%post -n kubelet
%systemd_post kubelet
if [ $1 -gt 1 ] ; then
  systemctl daemon-reload >/dev/null 2>&1
  systemctl try-restart kubelet >/dev/null 2>&1
fi

%preun -n kubelet
%systemd_preun kubelet

%postun -n kubelet
%systemd_postun_with_restart kubelet

%changelog

* Fri Feb 08 2019 Tom Cocozzello <tom.cocozzello@oracle.com> - 1.10.5-2.2.4
- Fix CVE-2018-1002105

* Wed Feb 06 2019 Tom Cocozzello <tom.cocozzello@oracle.com> - 1.10.5-2.1.4
- Fix Pipeline

* Wed Feb 06 2019 Muminul Islam <muminul.islam@oracle.com> - 1.10.5-2.1.3
- Bump the version to 1.10.5-2.1.3

* Wed Feb 06 2019 Muminul Islam <muminul.islam@oracle.com> - 1.10.5-2.1.2
- CVE-2019-6486

* Thu Jul 19 2018 Thomas Tanaka <thomas.tanaka@oracle.com> - 1.10.5-2.0.2
- [Orabug 28366224]

* Wed Jul 11 2018 Thomas Tanaka <thomas.tanaka@oracle.com> - 1.10.5-2.0.1
- [Orabug 28324540]
- Add tar and which to the kube-system image

* Mon Jun 25 2018 Thomas Tanaka <thomas.tanaka@oracle.com> - 1.10.5-1.0.2
- Kubelet RPM should install /usr/libexec/kubernetes/kubelet-plugins/

* Thu Jun 21 2018 Thomas Tanaka <thomas.tanaka@oracle.com> - 1.10.5-1.0.1
- Update to v1.10.5

* Tue Jun 19 2018 Thomas Tanaka <thomas.tanaka@oracle.com> - 1.10.4-1.0.1
- Update to v1.10.4

* Thu May 10 2018 Thomas Tanaka <thomas.tanaka@oracle.com> - 1.10.2-1.0.2.dev
- kubeadm-setup.sh join now changes for 1.10.2
- Fix pause-amd64 to be 3.1
- Upstream modification for Oracle

* Wed May 09 2018 Thomas Tanaka <thomas.tanaka@oracle.com> - 1.10.2-1.0.1.dev
- Update to v1.10.2

* Thu Apr 12 2018 Thomas Tanaka <thomas.tanaka@oracle.com> - 1.10.1-1.0.1.dev
- Update to v1.10.1
